#####################################################################
# The MakeRope.py script update the Magic Rope Setting for Blender 2.5 
# the script builds the rope with the Magic Rope setting.
# Authors: Maria Francesca Zini, Stefano Cianchetta from the SciVis group:  www.scivis.ifc.cnr.it
# The author of Original Magic Rope Setting is Philippe ROUBAL.  (http://www.3d-synthesis.com)
#####################################################################

import time, os, pickle, shutil, math
from bpy import *
#####################################################################
# Parameters
#####################################################################
#path where the "Data" folder is stored
pathlinux="E:/MakeRope2.5"
pathWin="C:\MagicRope\MagicRope2.5"
#Number of elements in rope:
ObjectNumber=30
#Elements type
#(uncomment the desired element and comment the undesired ones)

#objName="Sphere"
#objName="Cilinder"
objName="Caterpillar"
#objName="Garland"
#objName="Cartridge"
#objName="Chain"

#####################################################################
# End Paths
#####################################################################


def makeRope(verbose = False):

	# naive os detection
	if os.sep == "/":
		linux = True
	else:
		linux = False
	ops.object.select_all(action="DESELECT")
	context.scene.objects.active = None
	scene = context.scene
	try:
		# Append  object from external file 
		if linux:
			path =pathlinux+"/data/Ropelibrary.blend/Object/"
		else:
			path = pathWin+"\\data\\Ropelibrary.blend\\Object\\"
		ops.wm.link_append(filepath = path + objName, filename = objName, directory = path, link = False)
		data.objects[objName].name ="elementrope"
		context.scene.objects.active = data.objects["elementrope"]
		context.active_object.layers = context.scene.layers
	except Exception as E:
		raise Exception ("Template element object cannot be loaded from library: ", E)


	# Build 3D scene 
	curFrame=1
		# reset
	ops.object.select_all(action="DESELECT")
	scene.objects.active = None
	scene.frame_set(curFrame)

	data.objects["elementrope"].hide = False
	data.objects["elementrope"].select = True
	#duplicate the object to build the rope
	try:
		for i in range(ObjectNumber):
			ops.object.duplicate(linked = True, mode=1)
		print ("Making 5 empiets")
		ops.object.add(type='EMPTY', view_align=False, enter_editmode=False, location=(0.0, 0.0, 0.0), rotation=(0.0, 0.0, 0.0))
		for i in range(ObjectNumber+1):
			ops.object.duplicate(linked = True, mode=1)
	except Exception as E:
		raise Exception ("Unable to duplicate template", E)

	#walk through list of objects and set location
	try:
		counterels=0
		counterems=0
		for i, obj in enumerate(data.objects):
			if obj.name[0:11]=="elementrope":
				obj.location=[0,0,0]
				obj.location=[counterels*(-1.8),0,0]
				counterels=counterels+1
			if obj.name[0:5]=="Empty":
				if obj.name=="Empty":
					print("I'm scaling")
					ops.object.select_all(action="DESELECT")
					ops.object.select_name(name="Empty", extend=False)
					obj.scale[0]=10
					obj.scale[1]=10
					obj.scale[2]=10
					ops.object.scale_apply()
				obj.location=[0,0,0]
				obj.location=[counterems*(-1.8),0,0]
				counterems=counterems+1
	except Exception as E:
		raise Exception ("Unable to locate objects", E)
		
	#walk through list of objects and set parenting
	try:
		counterels=0
		counterems=0
		for i, obj in enumerate(data.objects):
			if obj.name[0:11]=="elementrope":
				if obj.name=="elementrope":
					ops.object.select_all(action="DESELECT")
					ops.object.select_name(name=obj.name, extend=False)
					ops.object.select_name(name="Empty", extend=True)
					ops.object.parent_set(type='OBJECT')
					emptytarget="Empty.001"
					obj.constraints["TrackTo"].target = data.objects[emptytarget]
				else:
					emptyname="Empty."+obj.name.split(".")[1]
					ops.object.select_all(action="DESELECT")
					print(obj.name,emptyname)
					ops.object.select_name(name=obj.name, extend=False)
					ops.object.select_name(name=emptyname, extend=True)
					ops.object.parent_set(type='OBJECT')
					emptytarget="Empty."+"%03d" % (int(obj.name.split(".")[1])+1)
					obj.constraints["TrackTo"].target = data.objects[emptytarget]
			if obj.name[0:5]=="Empty" and obj.name!="Empty":
				if obj.name=="Empty.001":
					ops.object.select_all(action="DESELECT")
					ops.object.select_name(name=obj.name, extend=False)
					ops.object.select_name(name="elementrope", extend=True)
					ops.object.parent_set(type='OBJECT')
				else:
					elementname="elementrope."+"%03d" % (int(obj.name.split(".")[1])-1)
					ops.object.select_all(action="DESELECT")
					ops.object.select_name(name=obj.name, extend=False)
					ops.object.select_name(name=elementname, extend=True)
					ops.object.parent_set(type='OBJECT')
	except Exception as E:
		raise Exception ("Unable to parent objects", E)


makeRope()